//3456789_123456789_123456789_123456789_123456789_123456789_123456789_123456789_
//
//
// quikMeshImg.cpp: implementation of the CquikMeshImg class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Proj1.h"
#include "quikMeshImg.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CquikMeshImg::CquikMeshImg()							
//------------------------------------------------------------------------------
// Constructor, initializer
{
	m_pTimg = NULL;							// no texture image,
	m_Txsize = m_Tysize = m_Tzsize = 0;		// so texture size is zero.
	m_Mxsize = m_Mysize = 2;				// default mesh; a single square.
	m_pNorm = NULL;							// No normal vector array exists,
	m_pTexy = NULL;							// No texture coord array exists,
	m_pVert = NULL;							// No vertex array exists.
}

CquikMeshImg::~CquikMeshImg()
//------------------------------------------------------------------------------
// Destructor (default)
{
	wipe();									// destroy contents, release memory
}

void CquikMeshImg::sizer(int imgX, int imgY, int meshX, int meshY)
//------------------------------------------------------------------------------
// Make our stored image size match (imgX,imgY) and stored mesh size match
// (meshX,meshY), resizing either one as necessary.
{
}

void CquikMeshImg::wipe(void)
//------------------------------------------------------------------------------
// Wipe out the contents of this object; free any memory it uses.
{
	if(m_pTimg!=NULL)						// ? do we have a texture image?
	{										// Yes... release that memory
		delete [] m_pTimg;					// we reserved in makeTestImage()
	}
	m_Txsize = m_Tysize = m_Tzsize = 0;		// zero size.
	if(m_pNorm!=NULL)						// ? do we have a normal vec. array?
	{
		delete [] m_pNorm;					// Yes... release the memory we
	}										// reserved in makeTestImage()
	if(m_pTexy!=NULL)						// ? do we have texture coord array?
	{
		delete [] m_pTexy;					// Yes... release the memory we
	}										// reserved in makeTestImage()
	if(m_pNorm!=NULL)						// ? do we have a vertex array?
	{
		delete [] m_pVert;					// Yes... release the memory we
	}										// reserved in makeTestImage()
	m_Txsize = m_Tysize = m_Tzsize = 0;		// set image size to zero.
	m_Mxsize = m_Mysize = 0;				// set mesh size to zero.
}
/*
void CquikMeshImg::makeTestImgMeshDisplayList(void)			
//------------------------------------------------------------------------------
// Generate a test pattern texture from color ramps in various directions.
// Builds a vertex array for mesh to allow easy modification & fast rendering.
{
int i,j,k;									
float xfrac,yfrac;

	m_Txsize = m_Tysize = 64;					// OpenGL requires power-of-two
												// sized, square texture maps
	m_Tzsize = 4;								// RGBA pixels (transparency!
	m_pTimg = new GLubyte[m_Txsize*m_Tysize*m_Tzsize];	// reserve memory.
	for(j=0,k=0; j<m_Tysize; j++)				
	{											// Make colorful test pattern:
		for(i=0; i<m_Txsize; i++)				// pixel i on scanline j
		{
			m_pTimg[(k + i)*m_Tzsize + 0] =((255-i)*7 +(    j)* 4)%256; //red
			m_pTimg[(k + i)*m_Tzsize + 1] =((255-i)*3 +(255-j)* 8)%256; //green
			m_pTimg[(k + i)*m_Tzsize + 2] =((255-i)*2 +(    j)*11)%256; //blue
			m_pTimg[(k + i)*m_Tzsize + 3] = 170;	// alpha (opacity)
		}
		k += m_Txsize;							// start of scanline 'j'
	}
												// (use default mesh size)
	xfrac = 1.0f/(float)(m_Mxsize-1);			// fractional mesh spacing
	yfrac = 1.0f/(float)(m_Mysize-1);
	//Make a display list for the mesh..........................................
	m_dList = glGenLists(1);				// Reserve a display list for us;
												// m_dList is its identifier#.
	glNewList(m_dList,GL_COMPILE);				// Start the list...
	glBegin(GL_QUADS);							// Make a quadrilateral mesh.
	for(j=0; j<m_Mysize-1; j++)					// height of mesh
	{
		for(i=0; i<m_Mxsize-1; i++)		
		{	// mesh spans 0.0 < x,y < 1.0, but texture coords span -1.0<x,y<2.0
			// so that texture image (0.0<x,y<1.0) is in middle third of the
			// mesh, and GL_TEXTURE_WRAP_S,T settings determine how to draw the
			// outer 2/3 of the image.
			// (Note; MUST call normal,texture,vertex IN THAT ORDER!)
			glNormal3f(0.0f,0.0f,1.0f);					// lower-left
			glTexCoord2f((i  )*xfrac*3.0f - 1.0f,(j  )*yfrac*3.0f - 1.0f);		
			glVertex3f  ((i  )*xfrac,			 (j  )*yfrac, 0.0f);	

			glNormal3f(0.0f,0.0f,1.0f);					// lower-right,
			glTexCoord2f((i+1)*xfrac*3.0f - 1.0f,(j  )*yfrac*3.0f - 1.0f);				
			glVertex3f	((i+1)*xfrac,			 (j  )*yfrac, 0.0f);	
			
			glNormal3f(0.0f,0.0f,1.0f);					// upper-right,
			glTexCoord2f((i+1)*xfrac*3.0f - 1.0f,(j+1)*yfrac*3.0f - 1.0f);				
			glVertex3f	((i+1)*xfrac,			 (j+1)*yfrac,0.0f);	
			
			glNormal3f(0.0f,0.0f,1.0f);					// upper left.
			glTexCoord2f((i  )*xfrac*3.0f - 1.0f,(j+1)*yfrac*3.0f - 1.0f);				
			glVertex3f	((i  )*xfrac,			 (j+1)*yfrac,0.0f);		
		}
	}
	glEnd();		// stop making quads. mesh is finished.
	glEndList();	// stop making display list; finished.
}
*/

void CquikMeshImg::makeTestImgMesh(void)			
//------------------------------------------------------------------------------
// Generate a test pattern texture from color ramps in various directions.
// Builds a vertex array for mesh to allow easy modification & fast rendering.
{
int i,j,k,vnum;									
float xfrac,yfrac;

	m_Txsize = m_Tysize = 64;					// OpenGL requires power-of-two
												// sized, square texture maps
	m_Tzsize = 4;								// RGBA pixels (transparency!
	m_pTimg = new GLubyte[m_Txsize*m_Tysize*m_Tzsize];	// reserve memory.
	for(j=0,k=0; j<m_Tysize; j++)				
	{											// Make colorful test pattern:
		for(i=0; i<m_Txsize; i++)				// pixel i on scanline j
		{
			m_pTimg[(k + i)*m_Tzsize + 0] =((255-i)*7 +(    j)* 4)%256; //red
			m_pTimg[(k + i)*m_Tzsize + 1] =((255-i)*3 +(255-j)* 8)%256; //green
			m_pTimg[(k + i)*m_Tzsize + 2] =((255-i)*2 +(    j)*11)%256; //blue
			m_pTimg[(k + i)*m_Tzsize + 3] = 170;	// alpha (opacity)
		}
		k += m_Txsize;							// start of scanline 'j'
	}
	//Make a vertex array for the mesh.........................................
	m_Mxsize = 23;								// Set a mesh size
	m_Mysize = 17;								// (default is 2,2).
												// Reserve memory for arrays:
	m_pNorm = new GLfloat [3*m_Mxsize*m_Mysize];	// for Normal vectors
	m_pTexy = new GLfloat [2*m_Mxsize*m_Mysize];	// for Texture coordinates
	m_pVert = new GLfloat [3*m_Mxsize*m_Mysize];	// for Vertex coordinates

	xfrac = 1.0f/(float)(m_Mxsize-1);			// fractional mesh spacing
	yfrac = 1.0f/(float)(m_Mysize-1);
												// Fill arrays w/ vertex info:
	for(j=0,k=0; j<m_Mysize; j++, k+=m_Mxsize)	// height of mesh
	{
		for(i=0; i<m_Mxsize; i++)				// width of mesh
		{	// mesh spans 0.0 < x,y < 1.0, but texture coords span -1.0<x,y<2.0
			// so that texture image (0.0<x,y<1.0) is in middle third of the
			// mesh, and GL_TEXTURE_WRAP_S,T settings determine how to draw the
			// outer 2/3 of the image.
			vnum = i+k;							// Vertex number
			m_pNorm[3*vnum+0]= 0.0f;				// Nx,Ny,Nz coords
			m_pNorm[3*vnum+1]= 0.0f; 
			m_pNorm[3*vnum+2]= 1.0f;
												// Tx, Ty coords
			// texture fills center 1/3rd of mesh
			m_pTexy[2*vnum+0]= i*xfrac*3.0f - 1.0f;	
			m_pTexy[2*vnum+1]= j*yfrac*3.0f - 1.0f;
			// texture fills mesh
			//	m_pTexy[2*vnum+0]= i*xfrac;		
			//	m_pTexy[2*vnum+1]= j*yfrac;

			m_pVert[3*vnum+0]= i*xfrac;			// Vx,Vy,Vz coords
			m_pVert[3*vnum+1]= j*yfrac;
			m_pVert[3*vnum+2]= 0.0f;	
/*			ASSERT(i+k < m_Mxsize*m_Mysize);
			TRACE("N=%f,%f,%f \t",m_pNorm[i+k],m_pNorm[i+k+1],m_pNorm[i+k+2]);
			TRACE("T=%f,%f \t",m_pTexy[i+k],m_pTexy[i+k+1]);
			TRACE("V=%f,%f%,%f\n",m_pVert[i+k],m_pVert[i+k+1],m_pVert[i+k+2]);
*/		}
	}
}

void CquikMeshImg::drawMe(void)
//------------------------------------------------------------------------------
// Call from within CquikGL::drawOpenGL() only. (Why? fcn assumes lots of openGL
// state info has already been set by CquikGL::OnCreate_GL(), and that we have
// a valid openGL rendering context attached to the Windows device context.
{
int i,j,k;

	glPushAttrib(GL_LIGHTING_BIT);			// save current lighting state
	glPushAttrib(GL_COLOR_BUFFER_BIT);		// save current transparency state
	glPushAttrib(GL_TEXTURE_BIT);			// save current texture state

	glDisable(GL_LIGHTING);					// turn off lights for axes,
	glEnable(GL_BLEND);						// enable tex/poly transparency
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glColor3f(1.0f,1.0f,1.0f);				// set untextured poly's color to
											// white to avoid modulating the
											// texture map's alpha values.

	//..Set openGL's texture map state params:  (s,t) are texture coords.
	//	WRAP== if texture coords (s,t) fall outside the texture image coords, 
	//	use CLAMP to stay within image, or GL_REPEAT to use image coords 
	// modulo img. size for repeat texture 'tiling'
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP); 
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	error_GL("drawOpenGL::tex wrap");
									// Set texture map antialiasing method: 
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
									// when magnifying, use bilinear interpol.
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
									// when reducing, I use bilinear filter
	// Some of the other choices: 
	//  GL_NEAREST, or GL_LINEAR == nearest neighbor, or bilinear filt. src img.
	//  GL_NEAREST_MIPMAP_LINEAR == nearest within 2 mipmaps; blend (default)
	//  GL_LINEAR_MIPMAP_NEAREST == linear interp. within the best mipmap
	//								(has 'resolution popping' in animation)
	//  GL_LINEAR_MIPMAP_LINEAR == trilinear interp. (slowest; looks best)
	//  glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_DECAL);
	//	glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_BLEND);
		glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
	// Set the way texture is applied to polygon.
	// Choices: 
	//		GL_DECAL (most common). Set poly's on-screen color from texture.
	//			texture alpha (if any) blends with lighted poly color.
	//		GL_MODULATE texture map modulates surface lighting (if no lighting
	//		and polygon color is (1,1,1,1), then texmap sets poly color.
	//		GL_BLEND Fakes using texture as diffuse reflectance for lit poly
	glTexImage2D(GL_TEXTURE_2D,	// Tell openGL which texture map to use;
		0,						// pyramid level to load; 0==base of pyramid);
		m_Tzsize,				// # of color components[1-4]  (3==RGB,4=RGBA)
		m_Txsize,				// width: must be 2^n +2*border 
		m_Tysize,				// height; must be 2^m + 2*border
		0,						// border; must be 0 or 1.
		GL_RGBA,				// pixel format (others include GL_RGB, etc.
		GL_UNSIGNED_BYTE,		// pixel element type.
		m_pTimg);				// ptr to the texture image to load.
	error_GL("drawOpenGL--TexImage2D");	// Report any mistakes we made
	glEnable(GL_TEXTURE_2D);		// Last of all, enable 2D texture mapping.
	error_GL("drawOpenGL::tex 2D");

	//..Make a lit rectangle we'll texture map......
	glPolygonMode(GL_FRONT, GL_FILL);		// Solid, front-facing polys,
	//	glDisable(GL_LIGHTING);					// without lighting complications,
	
	// Use vertex-describing arrays to render the textured mesh:
	glMatrixMode(GL_MODELVIEW);					// save current modelview...
	glPushMatrix();	
	glTranslatef(-0.5f,-0.5f,-1.0f);			// move to P2 image position
												// (Draw a mesh of X,Y verts 
												// using X-1,Y-1 quadrilaterals)
											
						// makeTestImgMesh() put all vertex data into arrays at
						// pointers. Tell GL how to use them:
	glEnableClientState(GL_NORMAL_ARRAY);		// Enable arrays of vertex data
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	glEnableClientState(GL_VERTEX_ARRAY);
												// Tell GL how to use them:
	glNormalPointer(    GL_FLOAT,0,m_pNorm);	// 3-vector,float,no stride
	glTexCoordPointer(2,GL_FLOAT,0,m_pTexy);	// 2-vector,float,no stride
	glVertexPointer(  3,GL_FLOAT,0,m_pVert);	// 3-vector,float,no stride.

//int z;

	glBegin(GL_QUADS);
	for(j=0,k=0; j<m_Mysize-1; j++, k+=m_Mxsize)// (k is vert# for column 0)
	{
		for(i=0; i<m_Mxsize-1; i++)
		{
			glArrayElement(i+k);				// lower left corner of a quad
			glArrayElement(i+k+1);				// lower right,
			glArrayElement(i+k+1+m_Mxsize);		// upper right
			glArrayElement(i+k  +m_Mxsize);		// upper left.
		}
	}
	glEnd();									// Done with the mes.
	glPopMatrix();						// restore original MODELVIEW matrix.

	glPopAttrib();						// restore original texturing state
	glPopAttrib();						// restore original transparency state
	glPopAttrib();						// restore original lighting state
}

void CquikMeshImg::error_GL(char *label)
//------------------------------------------------------------------------------
// If OpenGL had any errors, print string found at 'label' to report where it
// happened in our code, and then print the error report. Otherwise do nothing.
// (NOTE: the 'TRACE()' macro works just like the 'printf()' function, EXCEPT 
//    output appears at bottom of the VisualC++6.0 window on the 'debug' tab.)
{
GLenum errorno;								// OpenGL error code.

	errorno = glGetError();					
	if (errorno != GL_NO_ERROR)				// was there any unreported error?
	{										// YES: tell me about it.
		TRACE("%s had error: #(%d) %s\r\n", 
		label, errorno, gluErrorString(errorno)); // (give OpenGL's explanation)
	}
}

void CquikMeshImg::meshWrinkle(float zoffset)
//------------------------------------------------------------------------------
// Displace odd-numbered vertices in the +z direction by 'zoffset'. Not very
// useful, but demonstrates how to move mesh vertices within vertex array, and
// shows vertex locations.
{
	
}


void CquikMeshImg::LoadFile(OPENFILENAME *pName)
//------------------------------------------------------------------------------
// DON'T CALL THIS FUNCTION YOURSELF! the CquikGL class calls it in response
// to user input.
{

}

void CquikMeshImg::SaveFile(OPENFILENAME *pName)
//------------------------------------------------------------------------------
// DON'T CALL THIS FUNCTION YOURSELF! the CquikGL class calls it in response
// to user input.
{

}
